/**
 * The enum-like class that describes a value type.
 *
 * @template T The name of the value type.
 * @see {@link Shape.inputs}
 * @group Type Inference
 */
export declare class Type<T extends string = string> {
    /**
     * The name of the type.
     */
    readonly name: T;
    static readonly ARRAY: Type<"array">;
    static readonly BIGINT: Type<"bigint">;
    static readonly BOOLEAN: Type<"boolean">;
    static readonly DATE: Type<"date">;
    static readonly FUNCTION: Type<"function">;
    static readonly MAP: Type<"map">;
    static readonly NULL: Type<"null">;
    static readonly NUMBER: Type<"number">;
    static readonly OBJECT: Type<"object">;
    static readonly PROMISE: Type<"promise">;
    static readonly SET: Type<"set">;
    static readonly STRING: Type<"string">;
    static readonly SYMBOL: Type<"symbol">;
    static readonly UNDEFINED: Type<"undefined">;
    static readonly UNKNOWN: Type<"unknown">;
    private constructor();
    /**
     * Returns the type of the given value. If value is a type itself, it is returned as is.
     *
     * @param value The value to get type of.
     * @return The type of the value.
     */
    static of(value: unknown): Type;
    /**
     * @internal
     */
    toString(): string;
}
//# sourceMappingURL=Type.d.ts.map