import { Issue } from './types';
/**
 * An error thrown if parsing failed. Custom check callbacks, refinement predicates, converters, and fallback
 * functions can throw this error to notify that the operation has failed.
 *
 * @group Other
 */
export declare class ValidationError extends Error {
    /**
     * The array of issues that caused the error.
     */
    issues: Issue[];
    /**
     * Creates a new {@link ValidationError} instance.
     *
     * @param issues The array of issues that caused the validation error.
     * @param message The error message. If omitted then `issues` are converted to a string using
     * {@link ValidationError.formatIssues} and used as a message.
     */
    constructor(
    /**
     * The array of issues that caused the error.
     */
    issues: Issue[], message?: string);
    /**
     * The global function that stringifies issues as an error message, if a message is omitted when
     * {@link ValidationError} is instantiated.
     */
    static formatIssues: (issues: Issue[]) => string;
}
//# sourceMappingURL=ValidationError.d.ts.map