import type { AnyShape, DeepPartialProtocol, DeepPartialShape, Shape } from '../shape/Shape';
import { ApplyOperationsCallback, ApplyOptions, CheckResult, Issue, Ok, Operation, ParseOptions, Result } from '../types';
type Awaited<T> = T extends null | undefined ? T : T extends object & {
    then(fn: infer F, ...args: any): any;
} ? F extends (value: infer V, ...args: any) => any ? Awaited<V> : never : T;
export type Promisify<T> = Promise<Awaited<T>>;
export type Awaitable<T> = Awaited<T> extends T ? Promise<T> | T : T;
export declare const defaultApplyOptions: Readonly<ApplyOptions>;
export declare const defaultEarlyReturnApplyOptions: Readonly<ApplyOptions>;
export declare const INPUT: unique symbol;
export declare const OUTPUT: unique symbol;
export type INPUT = typeof INPUT;
export type OUTPUT = typeof OUTPUT;
export declare function nextNonce(): number;
/**
 * For test purposes only!
 */
export declare function resetNonce(): void;
export declare function ok<T>(value: T): Ok<T>;
export declare function isAsyncShapes(shapes: readonly AnyShape[] | null | undefined): boolean;
/**
 * Converts the shape to its deep partial alternative if shape implements {@link DeepPartialProtocol}, or returns
 * the shape as is.
 */
export declare function toDeepPartialShape<S extends AnyShape & Partial<DeepPartialProtocol<any>>>(shape: S): DeepPartialShape<S>;
/**
 * Calls `Shape._apply` or `Shape._applyAsync` depending on `Shape.isAsync`, and passes the result to `cb` after it
 * becomes available.
 */
export declare function applyShape<T>(shape: AnyShape, input: unknown, options: ApplyOptions, nonce: number, cb: (result: Result) => T): T | Promise<Awaited<T>>;
/**
 * Prepends a key to a path of each issue.
 */
export declare function unshiftIssuesPath(issues: Issue[], key: unknown): void;
export declare function concatIssues(issues: Issue[] | null, otherIssues: Issue[]): Issue[];
export declare function captureIssues(error: unknown): Issue[];
/**
 * Returns an error message that is composed of the captured issues and parsing options.
 */
export declare function getErrorMessage(issues: Issue[], input: unknown, options: ParseOptions | undefined): string | undefined;
/**
 * Copies operations from `baseShape` to `shape`.
 */
export declare function copyOperations<S extends Shape>(baseShape: Shape, shape: S): S;
/**
 * The callback that converts output and issues to a {@link Result}.
 */
export declare const applyOperations: ApplyOperationsCallback;
/**
 * Creates the callback that applies the operation and passes control to the next operation.
 *
 * @param operation The operation to apply.
 * @param next The callback that applies the next operation.
 * @param async If `true` then both operation and next operation are called asynchronously.
 * @returns The callback that applies the operation.
 */
export declare function composeApplyOperations(operation: Operation, next: ApplyOperationsCallback, async: boolean): ApplyOperationsCallback;
export declare function extractCheckResult(result: CheckResult): Result;
export declare function throwSyncUnsupported(): never;
export {};
//# sourceMappingURL=shapes.d.ts.map