import { SetShape } from '../shape/SetShape';
import { AnyShape } from '../shape/Shape';
import { Any, IssueOptions, Message } from '../types';
declare module '../core' {
    interface Messages {
        /**
         * @default "Must have the minimum size of %s"
         */
        'set.min': Message | Any;
        /**
         * @default "Must have the maximum size of %s"
         */
        'set.max': Message | Any;
    }
    interface SetShape<ValueShape extends AnyShape> {
        /**
         * Constrains the set size.
         *
         * @param size The minimum set size.
         * @param options The issue options or the issue message.
         * @returns The clone of the shape.
         * @group Plugin Methods
         * @plugin {@link plugin/set-essentials! plugin/set-essentials}
         */
        size(size: number, options?: IssueOptions | Message): this;
        /**
         * Constrains the minimum set size.
         *
         * @param size The minimum set size.
         * @param options The issue options or the issue message.
         * @returns The clone of the shape.
         * @group Plugin Methods
         * @plugin {@link plugin/set-essentials! plugin/set-essentials}
         */
        min(size: number, options?: IssueOptions | Message): this;
        /**
         * Constrains the maximum set size.
         *
         * @param size The maximum set size.
         * @param options The issue options or the issue message.
         * @returns The clone of the shape.
         * @group Plugin Methods
         * @plugin {@link plugin/set-essentials! plugin/set-essentials}
         */
        max(size: number, options?: IssueOptions | Message): this;
        /**
         * Constrains the {@link !Set Set} to contain at least one element.
         *
         * @param options The issue options or the issue message.
         * @returns The clone of the shape.
         * @group Plugin Methods
         * @plugin {@link plugin/set-essentials! plugin/set-essentials}
         */
        nonEmpty(options?: IssueOptions | Message): this;
    }
}
/**
 * Enhances {@link core!SetShape SetShape} with additional methods.
 */
export default function enableSetEssentials(ctor: typeof SetShape<any>): void;
//# sourceMappingURL=set-essentials.d.ts.map