import { ApplyOptions, IssueOptions, Message, Result } from '../types';
import { CoercibleShape } from './CoercibleShape';
/**
 * The shape of a constant value.
 *
 * @template Value The expected constant value.
 * @group Shapes
 */
export declare class ConstShape<Value> extends CoercibleShape<Value> {
    /**
     * The expected constant value.
     */
    readonly value: Value;
    /**
     * Returns `true` if an input is equal to the const value, or `false` otherwise.
     */
    protected _typePredicate: (input: unknown) => boolean;
    /**
     * Returns issues associated with an invalid input value type.
     */
    protected _typeIssueFactory: (input: unknown, options: ApplyOptions) => import("../types").Issue;
    /**
     * Creates a new {@link ConstShape} instance.
     *
     * @param value The expected value.
     * @param options The issue options or the issue message.
     * @template Value The expected value.
     */
    constructor(
    /**
     * The expected constant value.
     */
    value: Value, options?: IssueOptions | Message);
    protected _getInputs(): readonly unknown[];
    protected _coerce(input: unknown): Value;
    protected _apply(input: unknown, options: ApplyOptions, nonce: number): Result<Value>;
}
//# sourceMappingURL=ConstShape.d.ts.map