import { ReadonlyDict } from '../internal/objects';
import { ApplyOptions, IssueOptions, Message, Result } from '../types';
import { CoercibleShape } from './CoercibleShape';
/**
 * The shape of a value enumeration.
 *
 * @template Value The union of allowed enum values.
 * @group Shapes
 */
export declare class EnumShape<Value> extends CoercibleShape<Value> {
    /**
     * The array of allowed values, a const key-value mapping, or an TypeScript enum object.
     */
    readonly source: readonly Value[] | ReadonlyDict<Value>;
    /**
     * The array of unique enum values.
     */
    readonly values: readonly Value[];
    /**
     * Returns issues associated with an invalid input value type.
     */
    protected _typeIssueFactory: (input: unknown, options: ApplyOptions) => import("../types").Issue;
    /**
     * Creates a new {@link EnumShape} instance.
     *
     * @param source The array of allowed values, a const key-value mapping, or an enum object.
     * @param options The issue options or the issue message.
     * @template Value The union of allowed enum values.
     */
    constructor(
    /**
     * The array of allowed values, a const key-value mapping, or an TypeScript enum object.
     */
    source: readonly Value[] | ReadonlyDict<Value>, options?: IssueOptions | Message);
    protected _getInputs(): readonly unknown[];
    protected _coerce(input: unknown): Value;
    protected _apply(input: any, options: ApplyOptions, nonce: number): Result<Value>;
}
/**
 * Returns unique values of the enum. Source must contain key-value and value-key mapping to be considered a native
 * enum.
 */
export declare function getEnumValues(source: ReadonlyDict): any[];
//# sourceMappingURL=EnumShape.d.ts.map