import { ApplyOptions, IssueOptions, Message, Result } from '../types';
import { Shape } from './Shape';
/**
 * The shape of the class instance.
 *
 * @template Ctor The class constructor.
 * @group Shapes
 */
export declare class InstanceShape<Ctor extends new (...args: any) => any> extends Shape<InstanceType<Ctor>> {
    readonly ctor: Ctor;
    /**
     * Returns issues associated with an invalid input value type.
     */
    protected _typeIssueFactory: (input: unknown, options: ApplyOptions) => import("../types").Issue;
    /**
     * Creates a new {@link InstanceShape} instance.
     *
     * @param ctor The class constructor.
     * @param options The issue options or the issue message.
     * @template Ctor The class constructor.
     */
    constructor(ctor: Ctor, options?: IssueOptions | Message);
    protected _getInputs(): readonly unknown[];
    protected _apply(input: unknown, options: ApplyOptions, nonce: number): Result<InstanceType<Ctor>>;
}
//# sourceMappingURL=InstanceShape.d.ts.map