import { ApplyOptions, Issue, IssueOptions, Message, Result } from '../types';
import { AnyShape, DeepPartialProtocol, DeepPartialShape, Input, Output, Shape } from './Shape';
/**
 * Converts union to intersection.
 */
type Intersect<U extends AnyShape> = (U extends any ? (k: U) => void : never) extends (k: infer I) => void ? I extends Shape ? I : never : never;
type DeepPartialIntersectionShape<Shapes extends readonly AnyShape[]> = IntersectionShape<{
    [K in keyof Shapes]: DeepPartialShape<Shapes[K]>;
}>;
/**
 * The shape that requires an input to conform all given shapes.
 *
 * @template Shapes The array of shapes that comprise an intersection.
 * @group Shapes
 */
export declare class IntersectionShape<Shapes extends readonly AnyShape[]> extends Shape<Input<Intersect<Shapes[number]>>, Output<Intersect<Shapes[number]>>> implements DeepPartialProtocol<DeepPartialIntersectionShape<Shapes>> {
    /**
     * The array of shapes that comprise an intersection.
     */
    readonly shapes: Shapes;
    /**
     * The intersection constraint options or an issue message.
     */
    protected _options: IssueOptions | Message | undefined;
    /**
     * Returns issues associated with an invalid input value type.
     */
    protected _typeIssueFactory: (input: unknown, options: ApplyOptions) => Issue;
    /**
     * Creates a new {@link IntersectionShape} instance.
     *
     * @param shapes The array of shapes that comprise an intersection.
     * @param options The intersection constraint options or an issue message.
     * @template Shapes The array of shapes that comprise an intersection.
     */
    constructor(
    /**
     * The array of shapes that comprise an intersection.
     */
    shapes: Shapes, options?: IssueOptions | Message);
    at(key: unknown): AnyShape | null;
    deepPartial(): DeepPartialIntersectionShape<Shapes>;
    protected _isAsync(): boolean;
    protected _getInputs(): readonly unknown[];
    protected _apply(input: any, options: ApplyOptions, nonce: number): Result<Output<Intersect<Shapes[number]>>>;
    protected _applyAsync(input: any, options: ApplyOptions, nonce: number): Promise<Result<Output<Intersect<Shapes[number]>>>>;
    /**
     * Merge values and apply checks.
     */
    private _applyIntersection;
}
/**
 * Merges values into one, or returns {@link NEVER} if values are incompatible.
 */
export declare function mergeValues(a: any, b: any): any;
export {};
//# sourceMappingURL=IntersectionShape.d.ts.map