import { ApplyOptions, Issue, IssueOptions, Message, Result } from '../types';
import { CoercibleShape } from './CoercibleShape';
import { AnyShape, DeepPartialProtocol, DeepPartialShape, Input, OptionalDeepPartialShape, Output } from './Shape';
/**
 * The shape of a {@link !Map Map} instance.
 *
 * @template KeyShape The key shape.
 * @template ValueShape The value shape.
 * @group Shapes
 */
export declare class MapShape<KeyShape extends AnyShape, ValueShape extends AnyShape> extends CoercibleShape<Map<Input<KeyShape>, Input<ValueShape>>, Map<Output<KeyShape>, Output<ValueShape>>, [
    unknown,
    unknown
][]> implements DeepPartialProtocol<MapShape<DeepPartialShape<KeyShape>, OptionalDeepPartialShape<ValueShape>>> {
    /**
     * The key shape.
     */
    readonly keyShape: KeyShape;
    /**
     * The value shape.
     */
    readonly valueShape: ValueShape;
    /**
     * The issue options or the issue message.
     */
    protected _options: IssueOptions | Message | undefined;
    /**
     * Returns issues associated with an invalid input value type.
     */
    protected _typeIssueFactory: (input: unknown, options: ApplyOptions) => Issue;
    /**
     * Creates a new {@link MapShape} instance.
     *
     * @param keyShape The key shape.
     * @param valueShape The value shape.
     * @param options The issue options or the issue message.
     * @template KeyShape The key shape.
     * @template ValueShape The value shape.
     */
    constructor(
    /**
     * The key shape.
     */
    keyShape: KeyShape, 
    /**
     * The value shape.
     */
    valueShape: ValueShape, options?: IssueOptions | Message);
    at(key: unknown): AnyShape | null;
    deepPartial(): MapShape<DeepPartialShape<KeyShape>, OptionalDeepPartialShape<ValueShape>>;
    protected _isAsync(): boolean;
    protected _getInputs(): readonly unknown[];
    /**
     * Coerces a value to an array of Map entries.
     *
     * @param input A value to coerce.
     * @returns An array of entries, or {@link NEVER} if coercion isn't possible.
     */
    protected _coerce(input: unknown): [unknown, unknown][];
    protected _apply(input: any, options: ApplyOptions, nonce: number): Result<Map<Output<KeyShape>, Output<ValueShape>>>;
    protected _applyAsync(input: any, options: ApplyOptions, nonce: number): Promise<Result<Map<Output<KeyShape>, Output<ValueShape>>>>;
}
//# sourceMappingURL=MapShape.d.ts.map