import { Any, ApplyOptions, IssueOptions, Message, Result } from '../types';
import { CoercibleShape } from './CoercibleShape';
import { AllowShape, ReplaceShape } from './Shape';
/**
 * The shape of a number value.
 *
 * @group Shapes
 */
export declare class NumberShape extends CoercibleShape<number> {
    /**
     * Returns issues associated with an invalid input value type.
     */
    protected _typeIssueFactory: (input: unknown, options: ApplyOptions) => import("../types").Issue;
    /**
     * Creates a new {@link NumberShape} instance.
     *
     * @param options The issue options or the issue message.
     */
    constructor(options?: IssueOptions | Message);
    /**
     * Allows `NaN` as an input and output value.
     */
    nan(): AllowShape<this, number>;
    /**
     * Replaces an input `NaN` value with a default output value.
     *
     * @param defaultValue The value that is used instead of `NaN` in the output.
     */
    nan<T extends Any>(defaultValue: T): ReplaceShape<this, number, T>;
    protected _getInputs(): readonly unknown[];
    protected _apply(input: any, options: ApplyOptions, nonce: number): Result<number>;
}
//# sourceMappingURL=NumberShape.d.ts.map