import { INPUT, OUTPUT, Promisify } from '../internal/shapes';
import { ApplyOptions, IssueOptions, Message, Result } from '../types';
import { CoercibleShape } from './CoercibleShape';
import { AnyShape, DeepPartialProtocol, OptionalDeepPartialShape } from './Shape';
type InferPromise<ValueShape extends AnyShape | null, Leg extends INPUT | OUTPUT> = Promisify<ValueShape extends null | undefined ? any : ValueShape extends AnyShape ? ValueShape[Leg] : any>;
type DeepPartialPromiseShape<ValueShape extends AnyShape | null> = PromiseShape<ValueShape extends null | undefined ? null : ValueShape extends AnyShape ? OptionalDeepPartialShape<ValueShape> : null>;
/**
 * The shape of a {@link !Promise Promise} value.
 *
 * @template ValueShape The shape of the resolved value, or `null` if resolved value shouldn't be parsed.
 * @group Shapes
 */
export declare class PromiseShape<ValueShape extends AnyShape | null> extends CoercibleShape<InferPromise<ValueShape, INPUT>, InferPromise<ValueShape, OUTPUT>, Promise<any>> implements DeepPartialProtocol<DeepPartialPromiseShape<ValueShape>> {
    readonly valueShape: ValueShape;
    /**
     * The type constraint options or the type issue message.
     */
    protected _options: IssueOptions | Message | undefined;
    /**
     * Returns issues associated with an invalid input value type.
     */
    protected _typeIssueFactory: (input: unknown, options: ApplyOptions) => import("../types").Issue;
    /**
     * Creates a new {@link PromiseShape} instance.
     *
     * @param valueShape The shape of the resolved value, or `null` if resolved value shouldn't be parsed.
     * @param options The issue options or the issue message.
     * @template ValueShape The shape of the resolved value.
     */
    constructor(valueShape: ValueShape, options?: IssueOptions | Message);
    deepPartial(): DeepPartialPromiseShape<ValueShape>;
    protected _isAsync(): boolean;
    protected _getInputs(): readonly unknown[];
    protected _coerce(input: unknown): Promise<any>;
    protected _apply(input: any, options: ApplyOptions, nonce: number): Result<InferPromise<ValueShape, OUTPUT>>;
    protected _applyAsync(input: any, options: ApplyOptions, nonce: number): Promise<Result<InferPromise<ValueShape, OUTPUT>>>;
}
export {};
//# sourceMappingURL=PromiseShape.d.ts.map