import { INPUT, OUTPUT } from '../internal/shapes';
import { ApplyOptions, Issue, IssueOptions, Message, Result } from '../types';
import { AnyShape, DeepPartialProtocol, OptionalDeepPartialShape, Shape } from './Shape';
type InferRecord<KeyShape extends Shape<string, PropertyKey> | null, ValueShape extends AnyShape, Leg extends INPUT | OUTPUT> = Record<KeyShape extends null | undefined ? string : KeyShape extends Shape ? KeyShape[Leg] : string, ValueShape[Leg]>;
/**
 * The shape that describes an object with string keys and values that conform the given shape.
 *
 * @template KeyShape The key shape.
 * @template ValueShape The value shape.
 * @group Shapes
 */
export declare class RecordShape<KeyShape extends Shape<string, PropertyKey> | null, ValueShape extends AnyShape> extends Shape<InferRecord<KeyShape, ValueShape, INPUT>, InferRecord<KeyShape, ValueShape, OUTPUT>> implements DeepPartialProtocol<RecordShape<KeyShape, OptionalDeepPartialShape<ValueShape>>> {
    /**
     * The key shape or `null` if keys are preserved intact.
     */
    readonly keyShape: KeyShape;
    /**
     * The value shape.
     */
    readonly valueShape: ValueShape;
    /**
     * The issue options or the issue message.
     */
    protected _options: IssueOptions | Message | undefined;
    /**
     * Returns issues associated with an invalid input value type.
     */
    protected _typeIssueFactory: (input: unknown, options: ApplyOptions) => Issue;
    /**
     * Creates a new {@link RecordShape} instance.
     *
     * @param keyShape The key shape, or `null` if keys should be preserved intact.
     * @param valueShape The value shape.
     * @param options The issue options or the issue message.
     * @template KeyShape The key shape.
     * @template ValueShape The value shape.
     */
    constructor(
    /**
     * The key shape or `null` if keys are preserved intact.
     */
    keyShape: KeyShape, 
    /**
     * The value shape.
     */
    valueShape: ValueShape, options?: IssueOptions | Message);
    at(key: unknown): AnyShape | null;
    deepPartial(): RecordShape<KeyShape, OptionalDeepPartialShape<ValueShape>>;
    protected _isAsync(): boolean;
    protected _getInputs(): readonly unknown[];
    protected _apply(input: any, options: ApplyOptions, nonce: number): Result<InferRecord<KeyShape, ValueShape, OUTPUT>>;
    protected _applyAsync(input: any, options: ApplyOptions, nonce: number): Promise<Result<InferRecord<KeyShape, ValueShape, OUTPUT>>>;
}
export {};
//# sourceMappingURL=RecordShape.d.ts.map