import { ApplyOptions, Issue, IssueOptions, Message, Result } from '../types';
import { CoercibleShape } from './CoercibleShape';
import { AnyShape, DeepPartialProtocol, Input, OptionalDeepPartialShape, Output } from './Shape';
/**
 * The shape of a {@link !Set Set} instance.
 *
 * @template ValueShape The value shape.
 * @group Shapes
 */
export declare class SetShape<ValueShape extends AnyShape> extends CoercibleShape<Set<Input<ValueShape>>, Set<Output<ValueShape>>, unknown[]> implements DeepPartialProtocol<SetShape<OptionalDeepPartialShape<ValueShape>>> {
    /**
     * The value shape.
     */
    readonly valueShape: ValueShape;
    /**
     * The type constraint options or the type issue message.
     */
    protected _options: IssueOptions | Message | undefined;
    /**
     * Returns issues associated with an invalid input value type.
     */
    protected _typeIssueFactory: (input: unknown, options: ApplyOptions) => Issue;
    /**
     * Creates a new {@link SetShape} instance.
     *
     * @param valueShape The value shape.
     * @param options The issue options or the issue message.
     * @template ValueShape The value shape.
     */
    constructor(
    /**
     * The value shape.
     */
    valueShape: ValueShape, options?: IssueOptions | Message);
    at(key: unknown): AnyShape | null;
    deepPartial(): SetShape<OptionalDeepPartialShape<ValueShape>>;
    protected _isAsync(): boolean;
    protected _getInputs(): readonly unknown[];
    /**
     * Coerces a value to an array of unique values.
     *
     * @param input The value to coerce.
     * @returns An array, or {@link NEVER} if coercion isn't possible.
     */
    protected _coerce(input: unknown): unknown[];
    protected _apply(input: any, options: ApplyOptions, nonce: number): Result<Set<Output<ValueShape>>>;
    protected _applyAsync(input: any, options: ApplyOptions, nonce: number): Promise<Result<Set<Output<ValueShape>>>>;
}
//# sourceMappingURL=SetShape.d.ts.map